@@ -53,6 +53,7 @@ urlpatterns = patterns(
     url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)bundles/(?P<content_sha1>[0-9a-z]+)/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/\+remove-testrun-attribute$', 'test_run_remove_attribute'),
     url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)bundles/(?P<content_sha1>[0-9a-z]+)/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/result/(?P<relative_index>[0-9]+)/$', 'test_result_detail'),
     url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)bundles/(?P<content_sha1>[0-9a-z]+)/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/result/(?P<relative_index>[0-9]+)/\+update-comments$', 'test_result_update_comments'),
+    url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)bundles/(?P<content_sha1>[0-9a-z]+)/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/result/(?P<relative_index>[0-9]+)/\+update-units$', 'test_result_update_units'),
     url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)bundles/(?P<content_sha1>[0-9a-z]+)/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/result/(?P<relative_index>[0-9]+)/\+update-result-attribute$', 'test_result_update_attribute'),
     url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)bundles/(?P<content_sha1>[0-9a-z]+)/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/result/(?P<relative_index>[0-9]+)/\+remove-result-attribute$', 'test_result_remove_attribute'),
     url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)bundles/(?P<content_sha1>[0-9a-z]+)/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/hardware-context/$', 'test_run_hardware_context'),
@@ -74,6 +75,7 @@ urlpatterns = patterns(
     url(r'^image-charts/(?P<name>[a-zA-Z0-9-_]+)$', 'image_reports.views.image_report_display'),
     url(r'^image-charts/(?P<name>[a-zA-Z0-9-_]+)/\+detail$', 'image_reports.views.image_report_detail', name='image_report_detail'),
     url(r'^image-charts/(?P<name>[a-zA-Z0-9-_]+)/\+add-group$', 'image_reports.views.image_report_add_group', name='image_report_add_group'),
+    url(r'^image-charts/(?P<name>[a-zA-Z0-9-_]+)/\+select-group$', 'image_reports.views.image_report_select_group', name='image_report_select_group'),
     url(r'^image-charts/(?P<name>[a-zA-Z0-9-_]+)/\+order-update$', 'image_reports.views.image_report_order_update', name='image_report_order_update'),
     url(r'^image-charts/\+add$', 'image_reports.views.image_report_add'),
     url(r'^image-charts/(?P<name>[a-zA-Z0-9-_]+)/\+edit$', 'image_reports.views.image_report_edit'),
@@ -87,6 +89,7 @@ urlpatterns = patterns(
     url(r'^image-charts/(?P<name>[a-zA-Z0-9-_]+)/(?P<id>\d+)/\+settings-update$', 'image_reports.views.image_chart_settings_update'),
     url(r'^image-charts/\+filter-type-check$', 'image_reports.views.image_chart_filter_type_check'),
     url(r'^image-charts/\+get-chart-test$', 'image_reports.views.get_chart_test_data'),
+    url(r'^image-charts/\+get-group-names$', 'image_reports.views.get_group_names'),
     url(r'^image-charts/(?P<name>[a-zA-Z0-9-_]+)/(?P<id>\d+)/\+export$', 'image_reports.views.image_chart_export'),
     url(r'^image-charts/(?P<name>[a-zA-Z0-9-_]+)/(?P<id>\d+)/\+add-filter$', 'image_reports.views.image_chart_filter_add'),
     url(r'^image-charts/(?P<name>[a-zA-Z0-9-_]+)/(?P<id>\d+)/(?P<slug>\d+)$', 'image_reports.views.image_chart_filter_detail'),
