@@ -84,11 +84,14 @@ class ExpandedStatusColumn(tables.Column):
                              pklink(record.current_job),
                              record.current_job.description,
                              record.current_job.submitter))
-        elif record.status == Device.RESERVED:
-            return mark_safe("Reserved for job #%s - %s submitted by %s" % (
+        elif record.status == Device.RESERVED and record.current_job:
+            return mark_safe("Reserved for job #%s %s - %s submitted by %s" % (
                              pklink(record.current_job),
+                             record.current_job.status,
                              record.current_job.description,
                              record.current_job.submitter))
+        elif record.status == Device.RESERVED and not record.current_job:
+            return mark_safe("Reserved with <b>no current job</b>.")
         else:
             return Device.STATUS_CHOICES[record.status][1]
 
@@ -632,10 +635,10 @@ class DeviceTransitionTable(LavaTable):
     class Meta(LavaTable.Meta):
         model = DeviceStateTransition
         exclude = [
-            'id', 'device', 'job', 'old_state', 'new_state'
+            'device', 'job', 'old_state', 'new_state'
         ]
         sequence = [
-            'created_on', 'transition', 'created_by', 'message'
+            'id', 'created_on', 'transition', 'created_by', 'message'
         ]
         searches = {}
         queries = {}
