@@ -140,6 +140,12 @@ try:
 except ImportError:
     pass
 
+try:
+    import django_extensions
+    INSTALLED_APPS += ['django_extensions']
+except ImportError:
+    pass
+
 TEMPLATE_CONTEXT_PROCESSORS = [
     "django.contrib.auth.context_processors.auth",
     "django.core.context_processors.debug",
@@ -187,50 +193,3 @@ SESSION_SERIALIZER = 'django.contrib.sessions.serializers.PickleSerializer'
 
 # this is a tad ugly but the upstream package still needs something here.
 KEY_VALUE_STORE_BACKEND = 'db://lava_scheduler_app_devicedictionarytable'
-
-# Logging
-
-LOGGING = {
-    'version': 1,
-    'disable_existing_loggers': False,
-    'filters': {
-        'require_debug_false': {
-            '()': 'django.utils.log.RequireDebugFalse'
-        }
-    },
-    'formatters': {
-        'lava': {
-            'format': '%(levelname)s %(asctime)s %(module)s %(message)s'
-        }
-    },
-    'handlers': {
-        'logfile': {
-            'class': 'logging.handlers.WatchedFileHandler',
-            'filename': '/var/log/lava-server/django.log',
-            'formatter': 'lava'
-        }
-    },
-    'loggers': {
-        'django': {
-            'handlers': ['logfile'],
-            # DEBUG outputs all SQL statements
-            'level': 'ERROR',
-            'propagate': True,
-        },
-        'django_auth_ldap': {
-            'handlers': ['logfile'],
-            'level': 'INFO',
-            'propagate': True,
-        },
-        'dashboard_app': {
-            'handlers': ['logfile'],
-            'level': 'INFO',
-            'propagate': True,
-        },
-        'lava_scheduler_app': {
-            'handlers': ['logfile'],
-            'level': 'INFO',
-            'propagate': True,
-        }
-    }
-}
